#https://www.w3schools.com/python/python_datatypes.asp

data=[
    {'key': "<class 'str'>", "type":"str"},#
    {'key': "<class 'int'>", "type":"int"},#
    {'key': "<class 'float'>", "type":"float"},#
    {'key': "<class 'complex'>", "type":"complex"},#
    {'key': "<class 'bool'>", "type":"bool"},#
    {'key': "<class 'list'>", "type":"list"},#
    {'key': "<class 'tuple'>", "type":"tuple"},#
    {'key': "<class 'dict'>", "type":"dict"},#
    {'key': "<class 'set'>", "type":"set"},#
    {'key': "<class 'frozenset'>", "type":"frozenset"},#
    {'key': "<class 'bytes'>", "type":"bytes"},#
    {'key': "<class 'bytearray'>", "type":"bytearray"},#
    {'key': "<class 'memoryview'>", "type":"memoryview"},
    {'key': "<class 'NoneType'>", "type":"None"},#
    {'key': "class", "type":"class"}#
]




def find_type(type_object) -> str:
    """It returns the type in str format
    when doesn't find it, it returns an empty str"""

    key_str = str(type(type_object))
    for a in data:
        if a['key'] == key_str:
            return a['type']
    
    #detect class

    if key_str.find('class'):
        return data[14]['type']
    return ''

def str_wrapper(key_str:str, value:str, final_json = False):
    if key_str == '':
        return '"' + value + '"'
    res = '"' + key_str + '":"' + value + '"'
    if final_json:
        return final_wrapper(res)
    else:
        return res

def int_wrapper(key_str:str, value:int, final_json = False):
    if key_str == '':
        return str(value)
    res = '"' + key_str + '":' + str(value)
    if final_json:
        return final_wrapper(res)
    else:
        return res

def float_wrapper(key_str:str, value:float, final_json = False):
    if key_str == '':
        return str(value)
    res = '"' + key_str + '":' + str(value)
    if final_json:
        return final_wrapper(res)
    else:
        return res

def complex_wrapper(key_str:str, value:complex, final_json = False):
    if key_str == '':
        return '"' + str(value) + '"'
    res = '"' + key_str + '":"' + str(value) + '"'
    if final_json:
        return final_wrapper(res)
    else:
        return res

def bool_wrapper(key_str:str, value:bool, final_json = False):
    if key_str == '':
        return str(value)
    res = '"' + key_str + '":' + str(value)
    if final_json:
        return final_wrapper(res)
    else:
        return res

def list_wrapper(key_str:str,value:list,final_json = False):
    if key_str == '':
        res = '['
    else:
        res = '"' + key_str + '":['
    if len(value) > 0:
        for a in value:    
            res += jsonwhatever('',a,False) + ','
        res = res[:-1] + ']'
    else:
        res += ']'
    if final_json:
        return final_wrapper(res)
    else:
        return res

def dict_wrapper(key_str:str,value:dict,final_json = False):
    res = ''
    if len(value) > 0:
        for key in value:    
            value_dict = value[key]
            
            res += jsonwhatever(key_str=key,value=value_dict,final_json=False) + ','
        res = res[:-1]
    
    return final_wrapper(res)

def bytes_wrapper(key_str:str, value:bytes, final_json = False):
    if key_str == '':
        return str(value)
    res = '"' + key_str + '":' + '"' + str(value) + '"'
    if final_json:
        return final_wrapper(res)
    else:
        return res

def none_wrapper(key_str:str,final_json = False):
    res =  '"' + key_str + '":null'
    if final_json:
        return final_wrapper(res)
    else:
        return res

def class_wrapper(key_str:str,value:dict,final_json = False):
    return dict_wrapper(key_str=key_str,value=value.__dict__,final_json=False)

def final_wrapper(data:str):
    return '{' + data + '}'


def jsonwhatever(key_str:str, value, final_json = True):
    type_str = find_type(value)

    if type_str == data[0]['type']:#str
        return str_wrapper(key_str=key_str,value=value,final_json=final_json)

    if type_str == data[1]['type']:#int
        return int_wrapper(key_str=key_str,value=value,final_json=final_json)
    
    if type_str == data[2]['type']:#float
        return float_wrapper(key_str=key_str,value=value,final_json=final_json)
    
    if type_str == data[3]['type']:#complex
        return complex_wrapper(key_str=key_str,value=value,final_json=final_json)
    
    if type_str == data[4]['type']:#bool
        return bool_wrapper(key_str=key_str,value=value,final_json=final_json)

    if type_str == data[5]['type']:#list
        return list_wrapper(key_str=key_str,value=value,final_json=final_json)
    
    if type_str == data[6]['type']:#tuple
        return list_wrapper(key_str=key_str,value=list(value),final_json=final_json)

    if type_str == data[7]['type']:#dictionary
        return dict_wrapper(key_str=key_str,value=value,final_json=final_json)

    if type_str == data[8]['type']:#set
        return list_wrapper(key_str=key_str,value=list(value),final_json=final_json)
    
    if type_str == data[9]['type']:#fronzenset
        return list_wrapper(key_str=key_str,value=list(value),final_json=final_json)

    if type_str == data[10]['type']:#bytes
        return bytes_wrapper(key_str=key_str,value=value,final_json=final_json)
    
    if type_str == data[11]['type']:#bytearray
        return bytes_wrapper(key_str=key_str,value=bytes(value),final_json=final_json)
        
    if type_str == data[13]['type']:#nonetype
        return none_wrapper(key_str=key_str,final_json=final_json)

    if type_str == data[14]['type']:#object class
        return class_wrapper(key_str=key_str,value=value,final_json=final_json)
    


from jsonwhatever import jsonwhatever

thisdict = {
  "brand": "Ford",
  "model": "Mustang",
  "year": 1964
}

class Dog:
  def __init__(self) -> None:
    self.id = 0
    self.name = 'perro'
    self.size = 5.3

class Person:
  def __init__(self, id, name, dog) -> None:
    self.id = id
    self.name = name
    self.dog = dog

dog_a = Dog()

complex_number = 5+9j
list_b = [4,5,6,8]
list_a = [2,3,'hello',7,list_b]
list_c = [4,5,thisdict,8,complex_number]
empty_list = []
none_var = None
bool_var = True
set_example_empty = set()
set_example = {1,2,3,4}
class_example = Person(9,'juan',dog_a)
bytes_example = bytes(4)
bytearray_example = bytearray(4)

print(jsonwhatever('list_example',list_b))
print(jsonwhatever('name','john'))
print(jsonwhatever('size',1.7))
print(jsonwhatever('empty_list',empty_list))
print(jsonwhatever('none_example',none_var))
print(jsonwhatever('boolean',bool_var))
print(jsonwhatever('empty_set',set_example_empty))
print(jsonwhatever('set_example',set_example))
print(jsonwhatever('thisdict',thisdict))
print(jsonwhatever('person_class',class_example))
print(jsonwhatever('bytes_example',bytes_example))
print(jsonwhatever('bytearray_example',bytearray_example))
print(jsonwhatever('crazy_list',list_c))



from setuptools import setup, find_packages
import codecs
import os

VERSION = '0.0.2'
DESCRIPTION = 'A simple JSON string creator that takes whatever you have and transform it into a String'

with open("jsonwhatever/README.MD", "r",encoding="utf-8") as fh:
    long_description = fh.read()

# Setting up
setup(
    name='jsonwhatever',
    version=VERSION,
    author='dazjuancarlos',
    author_email='dazjuancarlos@gmail.com',
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    install_requires=[],
    license='dazjuancarlos',
    keywords=['python','json','simple'],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)